from sqlalchemy import create_engine, MetaData, Table
from sqlalchemy.orm import mapper, sessionmaker

class Analysis(object):
    pass

class AnalysisProperty(object):
    pass

class Organism(object):
    pass

class Dbxref(object):
    pass

class Cvterm(object):
    pass

class Db(object):
    pass

class Cv(object):
    pass

def ChadoAuth(parser):
    parser.add_argument('-o', '--dbhost', required=True, help='Database Host')
    parser.add_argument('-n', '--dbname', required=True, help='Database Name')
    parser.add_argument('-u', '--dbuser', help='Database Username')
    parser.add_argument('-p', '--dbpass', help='Database Password')
    parser.add_argument('--dbschema', help='Database Schema (default: public)', default="public")
    parser.add_argument("-d", "--debug", help="Print debug information", action="store_true")

class ChadoInstance(object):

    def __init__(self, dbhost, dbname, dbuser, dbpass, dbschema, debug):
        self.dbhost = dbhost
        self.dbname = dbname
        self.dbuser = dbuser
        self.dbpass = dbpass
        self.dbschema = dbschema

        self.debug = debug

    def __str__(self):
        return '<ChadoInstance at %s>' % self.dbhost

    def connect(self):

        self._engine = create_engine('postgresql://%s:%s@%s/%s' % (self.dbuser, self.dbpass, self.dbhost, self.dbname), echo=self.debug)
        self._metadata = MetaData(self._engine, schema=self.dbschema)

        Session = sessionmaker(bind=self._engine)
        self.session = Session()

        self._test_db_access()

        self._reflect_tables()

    def _test_db_access(self):
        tables = self._engine.table_names(schema=self.dbschema)
        if ('analysis' not in tables or 'feature' not in tables):
            raise Exception("Could not find Chado tables in db %s" % (self._engine.url))

    def _reflect_tables(self):

        # Try to reflect tables
        analysis = Table('analysis', self._metadata, autoload=True)
        mapper(Analysis, analysis)
        analysisprop = Table('analysisprop', self._metadata, autoload=True)
        mapper(AnalysisProperty, analysisprop)
        organism = Table('organism', self._metadata, autoload=True)
        mapper(Organism, organism)
        dbxref = Table('dbxref', self._metadata, autoload=True)
        mapper(Dbxref, dbxref)
        cvterm = Table('cvterm', self._metadata, autoload=True)
        mapper(Cvterm, cvterm)
        db = Table('db', self._metadata, autoload=True)
        mapper(Db, db)
        cv = Table('cv', self._metadata, autoload=True)
        mapper(Cv, cv)

    def get_cvterm_id(self, type_name, cv_name):
        res = self.session.query(Cvterm, Cv).filter(Cvterm.name == type_name, Cv.name == cv_name)
        if not res.count():
            raise Exception("Could not find a cvterm with name '%s' from cv ''%s' in the database %s" % (type_name, cv_name, ci._engine.url))

        return res.one().Cvterm.cvterm_id



#!/usr/bin/env python
import json
import argparse
from chado import ChadoAuth, ChadoInstance, Organism

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Create a new organism')

    parser.add_argument("--genus", required=True, help="The genus of the organism")
    parser.add_argument("--species", help="The species of the organism")
    parser.add_argument("--common", required=True, help="The common name of the organism")
    parser.add_argument("--abbr", required=True, help="The abbreviation of the organism")
    parser.add_argument("--description", help="The abbreviation of the organism")

    ChadoAuth(parser)
    args = parser.parse_args()

    ci = ChadoInstance(args.dbhost, args.dbname, args.dbuser, args.dbpass, args.dbschema, args.debug)

    ci.connect()

    # check if the organism exists
    res = ci.session.query(Organism).filter_by(common_name = args.common)

    if (res.count() > 0):
        raise Exception("Found a preexisting organism with the same attributes in the database %s" % (ci._engine.url))

    org = Organism()
    org.abbreviation = args.abbr
    org.genus = args.genus
    org.species = args.species
    org.common_name = args.common
    org.comment = args.description
    ci.session.add(org)
    ci.session.commit()

    print "New organism created with ID: %s" % org.organism_id



#!/usr/bin/env python
import json
import argparse
from datetime import datetime
from chado import ChadoAuth, ChadoInstance, Analysis

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Create a new analysis')

    parser.add_argument("--name", required=True, help="Analysis name")
    parser.add_argument("--program", required=True, help="Program name")
    parser.add_argument("--program-version", required=True, help="Program version")
    parser.add_argument("--algorithm", help="Algorithm name")
    parser.add_argument("--source-name", required=True, help="Source name")
    parser.add_argument("--source-version", help="Source version")
    parser.add_argument("--source-uri", help="Source URI")
    parser.add_argument("--date-executed", help="Date of execution of the analysis (format=YYYY-MM-DD, default=today)")

    ChadoAuth(parser)
    args = parser.parse_args()

    ci = ChadoInstance(args.dbhost, args.dbname, args.dbuser, args.dbpass, args.dbschema, args.debug)

    ci.connect()

    # check if the analysis exists
    res = ci.session.query(Analysis).filter_by(name = args.name)

    if res.count() > 0:
        raise Exception("Found a preexisting analysis with the same attributes in the database %s" % (ci._engine.url))

    date = datetime.today()
    if args.date_executed:
        date = datetime.strptime(args.date_executed, '%Y-%m-%d')

    newa = Analysis()
    newa.name = args.name
    newa.program = args.program
    newa.programversion = args.program_version
    newa.algorithm = args.algorithm
    newa.sourcename = args.source_name
    newa.sourceversion = args.source_version
    newa.sourceuri = args.source_uri
    newa.timeexecuted = date
    ci.session.add(newa)
    ci.session.commit()

    print "New analysis created with ID: %s" % newa.analysis_id
